@@ -32,20 +32,25 @@
 #include "gdm.h"
 #include "misc.h"
 #include "getvt.h"
-#include "gdmconfig.h"
 
-#if defined (__linux__) || defined (__FreeBSD__) || defined(__DragonFly__)
+#include "gdm-common.h"
+#include "gdm-daemon-config.h"
 
-#ifdef __linux__
+#if defined (GDM_USE_SYS_VT) || defined (GDM_USE_CONSIO_VT)
+
+#ifdef __sun
+#define GDMCONSOLEDEVICE "/dev/vt/0"
+#else
+#define GDMCONSOLEDEVICE "/dev/console"
+#endif
+
+#if defined (GDM_USE_SYS_VT)
 #include <sys/vt.h>
-#endif
-#if defined(__FreeBSD__) || defined(__DragonFly__)
+#elif defined (GDM_USE_CONSIO_VT)
 #include <sys/consio.h>
-#endif
 
-#if defined( __FreeBSD__) || defined(__DragonFly__)
 static const char*
-__itovty(int val)
+__itovty (int val)
 {
 	static char str[8];
 	char* next = str + sizeof (str) - 1;
@@ -62,13 +67,17 @@ __itovty(int val)
 static int
 open_vt (int vtno)
 {
-	char *vtname;
-	int fd;
+	char *vtname = NULL;
+	int fd = -1;
 
-#if defined (__linux__)
+#if defined (GDM_USE_SYS_VT)
+#ifdef __sun
+	vtname = g_strdup_printf ("/dev/vt/%d", vtno);
+#else
 	vtname = g_strdup_printf ("/dev/tty%d", vtno);
-#elif defined (__FreeBSD__) || defined(__DragonFly__)
-	vtname = g_strdup_printf ("/dev/ttyv%s", __itovty(vtno - 1));
+#endif
+#elif defined (GDM_USE_CONSIO_VT)
+	vtname = g_strdup_printf ("/dev/ttyv%s", __itovty (vtno - 1));
 #endif
 	do {
 		errno = 0;
@@ -78,12 +87,16 @@ open_vt (int vtno)
 #endif
 			   , 0);
 	} while G_UNLIKELY (errno == EINTR);
+
 	g_free (vtname);
+
 	return fd;
 }
 
+#if defined (GDM_USE_SYS_VT)
+
 static int 
-get_free_vt (int *vtfd)
+get_free_vt_sys (int *vtfd)
 {
 	int fd, fdv;
 	int vtno;
@@ -94,7 +107,8 @@ get_free_vt (int *vtfd)
 
 	do {
 		errno = 0;
-		fd = open ("/dev/console", O_WRONLY
+		fd = open (GDMCONSOLEDEVICE,
+			   O_WRONLY
 #ifdef O_NOCTTY
 			   |O_NOCTTY
 #endif
@@ -108,7 +122,7 @@ get_free_vt (int *vtfd)
 		return -1;
 	}
 
-	for (vtno = gdm_get_value_int (GDM_KEY_FIRST_VT), vtmask = 1 << (vtno-1);
+	for (vtno = gdm_daemon_config_get_value_int (GDM_KEY_FIRST_VT), vtmask = 1 << vtno;
 			vtstat.v_state & vtmask; vtno++, vtmask <<= 1);
 	if (!vtmask) {
 		VE_IGNORE_EINTR (close (fd));
@@ -124,15 +138,87 @@ get_free_vt (int *vtfd)
 	return vtno;
 }
 
+#elif defined (GDM_USE_CONSIO_VT)
+
+static int
+get_free_vt_consio (int *vtfd)
+{
+	int fd, fdv;
+	int vtno;
+	GList *to_close_vts = NULL, *li;
+
+	*vtfd = -1;
+
+	do {
+		errno = 0;
+		fd = open (GDMCONSOLEDEVICE,
+			   O_WRONLY
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+			   , 0);
+	} while G_UNLIKELY (errno == EINTR);
+	if (fd < 0)
+		return -1;
+
+	if ((ioctl (fd, VT_OPENQRY, &vtno) < 0) || (vtno == -1)) {
+		VE_IGNORE_EINTR (close (fd));
+		return -1;
+	}
+
+	fdv = open_vt (vtno);
+	if (fdv < 0) {
+		VE_IGNORE_EINTR (close (fd));
+		return -1;
+	}
+
+	while (vtno < gdm_daemon_config_get_value_int (GDM_KEY_FIRST_VT)) {
+		int oldvt = vtno;
+		to_close_vts = g_list_prepend (to_close_vts,
+					       GINT_TO_POINTER (fdv));
+
+		if (ioctl (fd, VT_OPENQRY, &vtno) == -1) {
+			vtno = -1;
+			goto cleanup;
+		}
+
+		if (oldvt == vtno) {
+			vtno = -1;
+			goto cleanup;
+		}
+
+		fdv = open_vt (vtno);
+		if (fdv < 0) {
+			vtno = -1;
+			goto cleanup;
+		}
+	}
+
+	*vtfd = fdv;
+
+cleanup:
+	for (li = to_close_vts; li != NULL; li = li->next) {
+		VE_IGNORE_EINTR (close (GPOINTER_TO_INT (li->data)));
+	}
+	return vtno;
+}
+
+#endif
+
 char *
 gdm_get_empty_vt_argument (int *fd, int *vt)
 {
-	if ( ! gdm_get_value_bool (GDM_KEY_VT_ALLOCATION)) {
+	if ( ! gdm_daemon_config_get_value_bool (GDM_KEY_VT_ALLOCATION)) {
 		*fd = -1;
 		return NULL;
 	}
 
-	*vt = get_free_vt (fd);
+#if defined (GDM_USE_SYS_VT)
+	*vt = get_free_vt_sys (fd);
+#elif defined (GDM_USE_CONSIO_VT)
+	*vt = get_free_vt_consio (fd);
+#endif
+
 	if (*vt < 0)
 		return NULL;
 	else
@@ -144,12 +230,14 @@ void
 gdm_change_vt (int vt)
 {
 	int fd;
+	int rc;
 	if (vt < 0)
 		return;
 
 	do {
 		errno = 0;
-		fd = open ("/dev/console", O_WRONLY
+		fd = open (GDMCONSOLEDEVICE,
+			   O_WRONLY
 #ifdef O_NOCTTY
 			   |O_NOCTTY
 #endif
@@ -158,8 +246,8 @@ gdm_change_vt (int vt)
 	if (fd < 0)
 		return;
 
-	ioctl (fd, VT_ACTIVATE, vt);
-	ioctl (fd, VT_WAITACTIVE, vt);
+	rc = ioctl (fd, VT_ACTIVATE, vt);
+	rc = ioctl (fd, VT_WAITACTIVE, vt);
 
 	VE_IGNORE_EINTR (close (fd));
 }
@@ -167,16 +255,17 @@ gdm_change_vt (int vt)
 int
 gdm_get_cur_vt (void)
 {
-#if defined (__linux__)
+#if defined (GDM_USE_SYS_VT)
 	struct vt_stat s;
-#elif defined (__FreeBSD__) || defined(__DragonFly__)
+#elif defined (GDM_USE_CONSIO_VT)
 	int vtno;
 #endif
 	int fd;
 
 	do {
 		errno = 0;
-		fd = open ("/dev/console", O_WRONLY
+		fd = open (GDMCONSOLEDEVICE,
+			   O_WRONLY
 #ifdef O_NOCTTY
 			   |O_NOCTTY
 #endif
@@ -184,7 +273,7 @@ gdm_get_cur_vt (void)
 	} while G_UNLIKELY (errno == EINTR);
 	if (fd < 0)
 		return -1;
-#if defined (__linux__)
+#if defined (GDM_USE_SYS_VT)
 	ioctl (fd, VT_GETSTATE, &s);
 
 	VE_IGNORE_EINTR (close (fd));
@@ -195,13 +284,13 @@ gdm_get_cur_vt (void)
 	*/
 
 	return s.v_active;
-#elif defined (__FreeBSD__) || defined(__DragonFly__)
+#elif defined (GDM_USE_CONSIO_VT)
 	if (ioctl (fd, VT_GETACTIVE, &vtno) == -1) {
 		VE_IGNORE_EINTR (close (fd));
 		return -1;
 	}
 
-	VE_IGNORE_EINTR (close (fd));
+VE_IGNORE_EINTR (close (fd));
 
 	/* debug */
 	/*
@@ -212,8 +301,10 @@ gdm_get_cur_vt (void)
 #endif
 }
 
-#else /* here this is just a stub, we don't know how to do this outside
-	 of linux really */
+#else /* GDM_USE_SYS_VT || GDM_USE_CONSIO_VT - Here this is just
+       * a stub, we do not know how to support this on other
+       * platforms
+       */
 
 char *
 gdm_get_empty_vt_argument (int *fd, int *vt)
