@@ -1,5 +1,7 @@
-/* GDM - The GNOME Display Manager
- * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This file Copyright (c) 2001 George Lebl
  *
@@ -19,14 +21,33 @@
  */
 
 #include "config.h"
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <locale.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "gdmlanguages.h"
+#include "gdm.h"
+#include "gdmwm.h"
+#include "gdmcommon.h"
 #include "gdmconfig.h"
+#include "gdmlanguages.h"
+
+#include "gdm-socket-protocol.h"
+
+#define LAST_LANGUAGE "Last"
+#define DEFAULT_LANGUAGE "Default"
+
+static GtkWidget    *tv                       = NULL;
+static GtkListStore *lang_model               = NULL;
+static GtkWidget    *dialog                   = NULL;
+static gchar        *current_language         = NULL;
+static gchar        *dialog_selected_language = NULL;
+static gint          savelang                 = GTK_RESPONSE_NO;
+static gboolean      always_restart           = FALSE;
+
+#include "gdm-common.h"
 
 typedef struct _Language Language;
 struct _Language {
@@ -441,7 +462,9 @@ gdm_lang_name (const char *language,
 	if (no_group) {
 		char *p = strchr (name, '|');
 		if (p != NULL) {
-			strcpy (name, p+1);
+			p = g_strdup (p + 1);
+			g_free (name);
+			name = p;
 		}
 	}
 
@@ -630,13 +653,6 @@ gdm_lang_read_locale_file (const char *locale_file)
 	if ( ! got_english)
 		langs = g_list_prepend (langs, g_strdup ("en_US"));
 
-	curlocale = setlocale (LC_MESSAGES, NULL);
-	if (curlocale != NULL &&
-	    strcmp (curlocale, "C") != 0 &&
-	    find_lang (curlocale, &clean) == NULL) {
-		langs = g_list_prepend (langs, g_strdup (curlocale));
-	}
-
 	langs = g_list_sort (langs, lang_collate);
 
 	VE_IGNORE_EINTR (fclose (langlist));
@@ -644,4 +660,489 @@ gdm_lang_read_locale_file (const char *locale_file)
 	return langs;
 }
 
-/* EOF */
+GtkListStore *
+gdm_lang_get_model (void)
+{
+   return lang_model;
+}
+
+void
+gdm_lang_initialize_model (gchar * locale_file)
+{
+  GList *list, *li;
+  GtkTreeIter iter;
+
+  list = gdm_lang_read_locale_file (locale_file);
+
+  lang_model = gtk_list_store_new (NUM_COLUMNS,
+				   G_TYPE_STRING,
+				   G_TYPE_STRING,
+				   G_TYPE_STRING);
+
+  gtk_list_store_append (lang_model, &iter);
+  gtk_list_store_set (lang_model, &iter,
+		      TRANSLATED_NAME_COLUMN, _("Last language"),
+		      UNTRANSLATED_NAME_COLUMN, NULL,
+		      LOCALE_COLUMN, LAST_LANGUAGE,
+		      -1);
+
+  gtk_list_store_append (lang_model, &iter);
+  gtk_list_store_set (lang_model, &iter,
+		      TRANSLATED_NAME_COLUMN, _("System Default"),
+		      UNTRANSLATED_NAME_COLUMN, NULL,
+		      LOCALE_COLUMN, DEFAULT_LANGUAGE,
+		      -1);
+
+  for (li = list; li != NULL; li = li->next)
+    {
+      char *lang = li->data;
+      char *name;
+      char *untranslated;
+
+      li->data = NULL;
+
+      if (!gdm_common_locale_is_displayable (lang)) {
+        g_free (lang);
+        continue;
+      }
+
+      name = gdm_lang_name (lang,
+			    FALSE /* never_encoding */,
+			    TRUE /* no_group */,
+			    FALSE /* untranslated */,
+			    FALSE /* markup */);
+
+      untranslated = gdm_lang_untranslated_name (lang,
+						 TRUE /* markup */);
+
+      gtk_list_store_append (lang_model, &iter);
+      gtk_list_store_set (lang_model, &iter,
+			  TRANSLATED_NAME_COLUMN, name,
+			  UNTRANSLATED_NAME_COLUMN, untranslated,
+			  LOCALE_COLUMN, lang,
+			  -1);
+
+      g_free (name);
+      g_free (untranslated);
+      g_free (lang);
+    }
+  g_list_free (list);
+}
+
+gint
+gdm_lang_get_savelang_setting (void)
+{
+  return savelang;
+}
+
+gchar *
+gdm_lang_check_language (const char *old_language)
+{
+  gchar *retval = NULL;
+
+  /* Don't save language unless told otherwise */
+  savelang = GTK_RESPONSE_NO;
+
+  if (old_language == NULL)
+    old_language = "";
+
+  /* If a different language is selected */
+  if (current_language != NULL && strcmp (current_language, LAST_LANGUAGE) != 0)
+    {
+      if (strcmp (current_language, DEFAULT_LANGUAGE) == 0)
+	retval = g_strdup ("");
+      else
+        retval = g_strdup (current_language);
+
+      /* User's saved language is not the chosen one */
+      if (strcmp (old_language, retval) != 0)
+	{
+	  gchar *primary_message;
+	  gchar *secondary_message;
+	  char *current_name, *saved_name;
+
+	  if (strcmp (current_language, DEFAULT_LANGUAGE) == 0)
+	    current_name = g_strdup (_("System Default"));
+	  else
+	    current_name = gdm_lang_name (current_language,
+					  FALSE /* never_encoding */,
+					  TRUE /* no_group */,
+					  TRUE /* untranslated */,
+					  TRUE /* markup */);
+	  if (strcmp (old_language, "") == 0)
+	    saved_name = g_strdup (_("System Default"));
+	  else
+	    saved_name = gdm_lang_name (old_language,
+					FALSE /* never_encoding */,
+					TRUE /* no_group */,
+					TRUE /* untranslated */,
+					TRUE /* markup */);
+
+	  primary_message = g_strdup_printf (_("Do you wish to make %s the default for future sessions?"),
+	                                     current_name);
+ 	  secondary_message = g_strdup_printf (_("You have chosen %s for this session, but your default setting is "
+	                                         "%s."), current_name, saved_name);
+	  g_free (current_name);
+	  g_free (saved_name);
+
+	  savelang = gdm_wm_query_dialog (primary_message, secondary_message,
+		_("Make _Default"), _("Just For _This Session"), TRUE);
+	  g_free (primary_message);
+	  g_free (secondary_message);
+	}
+    }
+  else
+    {
+      retval = g_strdup (old_language);
+    }
+
+  return retval;
+}
+
+static void
+selection_changed (GtkTreeSelection *selection,
+		   gpointer          data)
+{
+  GtkTreeIter iter;
+
+  if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+      g_free (dialog_selected_language);
+      gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter, LOCALE_COLUMN, &dialog_selected_language, -1);
+    }
+}
+
+static void
+tree_row_activated (GtkTreeView         *view,
+                    GtkTreePath         *path,
+                    GtkTreeViewColumn   *column,
+                    gpointer            data)
+{
+  GtkTreeIter iter;
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (lang_model), &iter, path))
+    {
+      g_free (dialog_selected_language);
+      gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter,
+			  LOCALE_COLUMN, &dialog_selected_language,
+			  -1);
+      gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  }
+}
+
+static void
+gdm_lang_setup_treeview (void)
+{
+  if (dialog == NULL)
+    {
+      GtkWidget *main_vbox;
+      GtkWidget *button;
+      GtkWidget **tmp_p;
+      GtkWidget *swindow;
+      GtkWidget *label;
+      char *s;
+
+      dialog = gtk_dialog_new_with_buttons (_("Select a Language"),
+#ifdef TODO
+					    GTK_WINDOW (parent_window),
+#endif
+					    NULL,
+					    0,
+					    GTK_STOCK_CANCEL,
+					    GTK_RESPONSE_CANCEL,
+					    NULL);
+					    
+      button = gtk_button_new_with_mnemonic (_("Change _Language"));
+      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+      gtk_widget_show (button);
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
+                                    GTK_RESPONSE_OK);
+					    
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+      gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+
+      main_vbox = gtk_vbox_new (FALSE, 6);
+      gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 5);
+      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+                          main_vbox, TRUE, TRUE, 0);
+  
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+				       GTK_RESPONSE_OK);
+      /* evil gcc warnings */
+      tmp_p = &dialog;
+      g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *)tmp_p);
+      s = g_strdup (_("_Select the language for your session to use:"));
+      label = gtk_label_new_with_mnemonic (s);
+      gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+      g_free (s);
+      gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+      gtk_box_pack_start (GTK_BOX (main_vbox),
+			  label, FALSE, FALSE, 0);
+      tv = gtk_tree_view_new ();
+      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
+      gtk_label_set_mnemonic_widget (GTK_LABEL (label), tv);
+      /* FIXME: we should handle this better, but things really look
+       * bad if we aren't always LTR */
+      gtk_widget_set_direction (tv, GTK_TEXT_DIR_LTR);
+      gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tv), FALSE);
+      gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
+					       GTK_DIALOG_MODAL,
+					       NULL,
+					       gtk_cell_renderer_text_new (),
+					       "text", TRANSLATED_NAME_COLUMN,
+					       NULL);
+      gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
+					       GTK_DIALOG_MODAL,
+					       NULL,
+					       gtk_cell_renderer_text_new (),
+					       "markup",
+					       UNTRANSLATED_NAME_COLUMN,
+					       NULL);
+      swindow = gtk_scrolled_window_new (NULL, NULL);
+      gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swindow), GTK_SHADOW_IN);
+      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+				  GTK_POLICY_AUTOMATIC,
+				  GTK_POLICY_AUTOMATIC);
+      gtk_container_add (GTK_CONTAINER (swindow), tv);
+      gtk_box_pack_start (GTK_BOX (main_vbox),
+			  swindow, TRUE, TRUE, 0);
+      gtk_window_set_default_size (GTK_WINDOW (dialog),
+				   MIN (400, gdm_wm_screen.width),
+				   MIN (600, gdm_wm_screen.height));
+      g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (tv))),
+			"changed",
+			(GCallback) selection_changed,
+			NULL);
+      g_signal_connect (G_OBJECT (tv),
+                        "row_activated",
+                        (GCallback) tree_row_activated,
+                        NULL);
+      gtk_tree_view_set_model (GTK_TREE_VIEW (tv),
+			       GTK_TREE_MODEL (lang_model));
+    }
+}
+
+gint
+gdm_lang_ask_restart (gchar *language)
+{
+	GtkWidget *dialog;
+	gchar *firstmsg;
+	gchar *secondmsg;
+	gchar *login;
+	gint response = GTK_RESPONSE_NO;
+
+	if (always_restart)
+		return GTK_RESPONSE_YES;
+
+	login = _("the login screen");
+	firstmsg = g_strdup_printf (_("Do you wish to restart %s with the chosen language?"),
+	                            login);
+	secondmsg = g_strdup_printf (_("You will restart %s with the %s locale."),
+	                             login,
+	                             language);
+
+	response = gdm_wm_query_dialog (firstmsg, secondmsg, _("_Yes"), _("_No"), FALSE);
+
+	g_free (firstmsg);
+	g_free (secondmsg);
+
+	return response;
+}
+
+static gboolean
+gdm_lang_get_restart_state (void)
+{
+  return  always_restart;
+}
+
+static void
+gdm_lang_set_restart_state (gboolean do_restart)
+{
+  always_restart = do_restart;
+}
+
+static void
+gdm_lang_restart_handler (GtkMenuItem *menu_item, gpointer user_data)
+{
+  if ((int)user_data == ALWAYS_RESTART)
+    always_restart = TRUE;
+  else
+    always_restart = FALSE;
+}
+
+static gchar *
+gdm_lang_get (void)
+{
+   return (current_language);
+}
+
+void
+gdm_lang_set_restart_dialog (char *language)
+{
+   /*
+    * Don't do anything if the language is already set to
+    * this value.
+    */
+   if (current_language == NULL ||
+      (current_language != NULL &&
+       strcmp (current_language, language) != 0))
+     {
+       gint response = GTK_RESPONSE_YES;
+
+       if (strcmp (language, LAST_LANGUAGE))
+         response = gdm_lang_ask_restart (language);
+
+       gdm_lang_set (language);
+
+       if (strcmp (language, LAST_LANGUAGE) &&
+          (response == GTK_RESPONSE_YES))
+         {
+           printf ("%c%c%c%c%s\n", STX,
+                   BEL,
+                   GDM_INTERRUPT_SELECT_LANG,
+                   response == GTK_RESPONSE_YES ? 1 : 0,
+                   language);
+           fflush (stdout);
+
+         }
+     }
+}
+
+void
+gdm_lang_set (char *language)
+{
+   char *locale;
+   GtkTreeSelection *selection;
+   GtkTreeIter iter = {0};
+
+   g_free (current_language);
+   current_language = g_strdup (language);
+
+   if (dialog == NULL)
+     gdm_lang_setup_treeview ();
+
+   if (language == NULL)
+      return;
+ 
+   lang_set_custom_callback (language);
+
+   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
+   gtk_tree_selection_unselect_all (selection);
+
+   if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (lang_model), &iter)) {
+      do {
+         gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter, LOCALE_COLUMN, &locale, -1);
+         if (locale != NULL && strcmp (locale, language) == 0) {
+            GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (lang_model), &iter);
+
+            gtk_tree_selection_select_iter (selection, &iter);
+            gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tv), path, NULL, FALSE, 0.0, 0.0);
+            gtk_tree_path_free (path);
+            break;
+         }
+      } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (lang_model), &iter));
+   }
+}
+
+/*
+ * The button with this handler appears in the F10 menu, so it
+ * cannot depend on callback data being passed in.
+ */
+void
+gdm_lang_handler (gpointer user_data)
+{
+  if (dialog == NULL)
+    gdm_lang_setup_treeview ();
+
+  gtk_widget_show_all (dialog);
+  gdm_wm_center_window (GTK_WINDOW (dialog));
+
+  gdm_wm_no_login_focus_push ();
+  if (tv != NULL)
+    {
+      GtkTreeSelection *selection;
+	  
+      gtk_widget_show_now (dialog);
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
+      if (selection == NULL)
+	gtk_tree_selection_select_path (selection, gtk_tree_path_new_first ());
+      else
+        {
+          GtkTreeIter iter;
+          GtkTreePath *path;
+          GtkTreeModel *tm = GTK_TREE_MODEL (lang_model);
+
+          gtk_tree_selection_get_selected (selection, &tm, &iter);
+          path = gtk_tree_model_get_path (GTK_TREE_MODEL (lang_model), &iter);
+          gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tv), path, NULL, FALSE, 0.0, 0.0);
+          gtk_tree_path_free (path);
+        }
+    }
+  switch (gtk_dialog_run (GTK_DIALOG (dialog)))
+    {
+    case GTK_RESPONSE_OK:
+      if (dialog_selected_language)
+        gdm_lang_set_restart_dialog (dialog_selected_language);
+
+      break;
+    case GTK_RESPONSE_CANCEL:
+    default:
+      break;
+    }
+
+  gdm_wm_no_login_focus_pop ();
+
+  if (dialog)
+    gtk_widget_hide (dialog);
+}
+
+void
+gdm_lang_op_lang (const gchar *args)
+{
+  gchar *language = gdm_lang_check_language (args);
+
+  if (gdm_lang_get_savelang_setting () == GTK_RESPONSE_CANCEL)
+    printf ("%c%s\n", STX, GDM_RESPONSE_CANCEL);
+  else
+    printf ("%c%s\n", STX, language);
+
+  fflush (stdout);
+  g_free (language);
+}
+
+void
+gdm_lang_op_slang (const gchar *args)
+{
+  if (gdm_lang_get_savelang_setting () == GTK_RESPONSE_YES)
+    printf ("%cY\n", STX);
+  else
+    printf ("%c\n", STX);
+
+  fflush (stdout);
+}
+
+void
+gdm_lang_op_setlang (const gchar *args)
+{
+  if (args)
+    gdm_lang_set ((char*)args);
+
+  printf ("%c\n", STX);
+  fflush (stdout);
+}
+
+void
+gdm_lang_op_always_restart (const gchar *args)
+{
+  if (args)
+    {
+      if (g_ascii_strcasecmp (args, "y") == 0)
+        gdm_lang_set_restart_state (TRUE);
+      else if (g_ascii_strcasecmp (args, "n") == 0)
+        gdm_lang_set_restart_state (FALSE);
+    }
+
+  printf ("%c\n", STX);
+  fflush (stdout);
+}
+
