@@ -79,6 +79,7 @@ gboolean GdmConfiguratorFound  = FALSE;
 
 /* FIXME: hack */
 GreeterItemInfo *welcome_string_info = NULL;
+GreeterItemInfo *root = NULL;
 
 extern gboolean session_dir_whacked_out;
 extern gboolean require_quarter;
@@ -97,15 +98,22 @@ greeter_ignore_buttons (gboolean val)
    ignore_buttons = val;
 }
 
-/* If in random theme mode then grab a random theme from those selected */
+static char *get_theme_file (const char *in, char **theme_dir);
+
+/* If in random theme mode then grab a random theme from those selected.
+ * If a theme doesn't exist, try  the rest of the list from the randomly chosen
+ * position for one that does.
+ */
 static char *
 get_random_theme ()
 {
     char **vec;
-	char *themes_list;
-	char *theme;
+    char *themes_list;
+    char *theme = NULL;
+    char *dir;
     int size;
-	int i;
+    int chosen;
+    int i;
 
     themes_list = gdm_config_get_string (GDM_KEY_GRAPHICAL_THEMES);
 
@@ -120,9 +128,26 @@ get_random_theme ()
     for (size = 0; vec[size] != NULL; size++) {}
 
 	/* Get Random Theme from list */
-	srand (time (NULL));
-	i = rand () % size;
-	theme = g_strdup (vec[i]);
+	srand ( time(NULL) );
+	chosen = rand() % size;
+
+	/* Find a theme that exists */
+	for (i = 0; i < size; i++) {
+		int j;
+		char *file;
+
+		j = (i + chosen >= size) ? i+chosen-size : i+chosen;
+		file = get_theme_file( vec[j], &dir );
+
+		g_free(file);
+		if (g_file_test(dir, G_FILE_TEST_IS_DIR)) {
+		    g_free(dir);
+			theme = g_strdup (vec[j]);
+			break;
+		}
+	    g_free(dir);
+	}
+
     g_strfreev (vec);
 
     return theme;
@@ -449,6 +474,18 @@ process_operation (guchar       op_code,
 
 	gdk_flush ();
 
+	if (greeter_show_only_background (root)) {
+		GdkPixbuf *background;
+		int width, height;
+
+		gtk_window_get_size (GTK_WINDOW (window), &width, &height);
+		background = gdk_pixbuf_get_from_drawable (NULL, gtk_widget_get_root_window(window), NULL, 0, 0, 0, 0 ,width, height);
+		if (background) {
+			gdm_common_set_root_background (background);
+			g_object_unref (background);
+		}
+	}
+
 	printf ("%c\n", STX);
 	fflush (stdout);
 
@@ -555,7 +592,7 @@ key_press_event (GtkWidget *widget, GdkEventKey *key, gpointer data)
 {
   if (DOING_GDM_DEVELOPMENT && (key->keyval == GDK_Escape))
     {
-      gtk_main_quit ();
+      process_operation (GDM_QUIT, NULL);
       
       return TRUE;
     }
@@ -822,6 +859,7 @@ static void
 gdm_read_config (void)
 {
 	gint i;
+	gchar *key_string = NULL;
 
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
@@ -884,23 +922,25 @@ gdm_read_config (void)
 	gdm_config_get_bool   (GDM_KEY_BROWSER);
 
 	/* Keys for custom commands */
-	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
-		gchar * key_string = NULL;
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
 		gdm_config_get_string (key_string);
-
+		g_free (key_string);
+		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
 		gdm_config_get_string (key_string);
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
 		gdm_config_get_string (key_string);
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
 		gdm_config_get_string (key_string);
-        
+		g_free (key_string);
+		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
 		gdm_config_get_string (key_string);
-
 		g_free (key_string);
 	}     
 
@@ -916,7 +956,8 @@ greeter_reread_config (int sig, gpointer data)
 {
 	gint i;
 	gboolean custom_changed = FALSE;
-
+	gchar *key_string = NULL;
+	
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
 
@@ -978,28 +1019,30 @@ greeter_reread_config (int sig, gpointer data)
 		_exit (DISPLAY_RESTARTGREETER);
 	}
 
-	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
-		gchar *key_string = NULL;
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
 		if (gdm_config_reload_string (key_string))
 			custom_changed = TRUE;
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
 		if (gdm_config_reload_string (key_string))
 			custom_changed = TRUE;
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
 		if (gdm_config_reload_string (key_string))
 			custom_changed = TRUE;
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
 		if (gdm_config_reload_string (key_string))
 			custom_changed = TRUE;
+		g_free (key_string);
 		
 		key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
 		if (gdm_config_reload_string (key_string))
 			custom_changed = TRUE;
-
 		g_free (key_string);
 	}     	
 	
@@ -1188,7 +1231,6 @@ main (int argc, char *argv[])
   sigset_t mask;
   GIOChannel *ctrlch;
   GError *error;
-  GreeterItemInfo *root;
   char *theme_file;
   char *theme_dir;
   gchar *gdm_graphical_theme;
@@ -1196,6 +1238,7 @@ main (int argc, char *argv[])
   guint sid;
   int r;
   gint i;
+  gchar *key_string = NULL;
 
   if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
     DOING_GDM_DEVELOPMENT = TRUE;
@@ -1336,7 +1379,6 @@ main (int argc, char *argv[])
 
   GdmCustomCmdsFound = g_new0 (gboolean, GDM_CUSTOM_COMMAND_MAX);
   for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
-	  gchar * key_string = NULL;
 	  /*  For each possible custom command */      
 	  key_string = g_strdup_printf("%s%d=", GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
 	  GdmCustomCmdsFound[i] = gdm_working_command_exists (gdm_config_get_string (key_string));
